@@ -3,6 +3,8 @@ from random import randint
 from string import ascii_letters
 
 def mktemp(tempdir):
+    if not os.path.isdir(tempdir):
+        os.mkdir(tempdir)
     return os.path.join(tempdir, _random_file_name())
 
 def _random_file_name():
