@@ -45,7 +45,7 @@ class Warehouse:
     def __repr__(self):
         return 'Warehouse(%s)' % repr(self.cachedir)
 
-    def __init__(self, cachedir, serializer = pickle, mutable = True, tempdir = None):
+    def __init__(self, cachedir, serializer = pickle, mutable = True, tempdir = None, memcache = False):
         self.cachedir = cachedir
         self.serializer = serializer
         self.mutable = mutable
@@ -55,9 +55,19 @@ class Warehouse:
             os.makedirs(self.tempdir)
         except FileExistsError:
             pass
+        if not memcache:
+            self.memcache = None
+        else:
+            self.memcache = {}
+            for key, value in zip(self.keys(), self.values()):
+                self.memcache[key] = value
 
     def filename(self, index):
-        return os.path.join(self.cachedir, *parse_identifier(index))
+        subpath = parse_identifier(index)
+        if subpath == []:
+            raise KeyError('You specified an empty key.')
+        else:
+            return os.path.join(self.cachedir, *subpath)
 
     def __iter__(self):
         return (k for k in self.keys())
@@ -65,17 +75,27 @@ class Warehouse:
     def __setitem__(self, index, obj):
         fn = self.filename(index)
         mkdir(fn)
-        if (not self.mutable) and os.path.exists(fn):
+        if (not self.mutable) and ((self.memcache != None and fn in self.memcache) or os.path.exists(fn)):
             raise PermissionError('This warehouse is immutable, and %s already exists.' % fn)
         else:
             tmp = mktemp(self.tempdir)
             with open(tmp, 'wb') as fp:
                 self.serializer.dump(obj, fp)
             os.rename(tmp, fn)
+            if self.memcache != None:
+                self.memcache[fn] = obj
 
     def __getitem__(self, index):
         fn = self.filename(index)
 
+        if self.memcache == None:
+            return self._get_fn(fn)
+        else:
+            if fn not in self.memcache and index in self._keys():
+                self.memcache[fn] = self._get_fn(fn)
+            return self.memcache[fn]
+
+    def _get_fn(self, fn):
         try:
             mtime_before = os.path.getmtime(fn)
         except OSError:
@@ -97,22 +117,33 @@ class Warehouse:
         if not self.mutable:
             raise PermissionError('This warehouse is immutable, so you can\'t delete things.')
 
-        path = self.filename(index)
+        fn = self.filename(index)
         try:
-            os.remove(path)
+            os.remove(fn)
         except DeleteError as e:
             raise KeyError(*e.args)
         else:
-            for path in _reversed_directories(self.cachedir, os.path.split(path)[0]):
-                if os.listdir(path) == []:
-                    os.rmdir(path)
+            for fn in _reversed_directories(self.cachedir, os.path.split(fn)[0]):
+                if os.listdir(fn) == []:
+                    os.rmdir(fn)
                 else:
                     break
+            if self.memcache != None and fn in self.memcache:
+                del(self.memcache[fn])
 
     def __contains__(self, index):
-        return os.path.isfile(self.filename(index))
+        fn = self.filename(index)
+        if self.memcache == None:
+            return os.path.isfile(fn)
+        else:
+            if fn not in self.memcache and os.path.isfile(fn):
+                self.memcache[fn] = self._get_fn(fn)
+            return fn in self.memcache
 
     def __len__(self):
+        if self.memcache != None:
+            return len(self.memcache)
+
         length = 0
         for dirpath, _, filenames in os.walk(self.cachedir):
             for filename in filenames:
@@ -120,23 +151,24 @@ class Warehouse:
         return length
 
     def keys(self):
+        return self._keys() if self.memcache == None else map(os.path.relpath, self.memcache.keys())
+
+    def _keys(self):
         for dirpath, _, filenames in os.walk(self.cachedir):
             for filename in filenames:
                 yield os.path.relpath(os.path.join(dirpath, filename), self.cachedir)
 
     def values(self):
-        for dirpath, _, filenames in os.walk(self.cachedir):
-            for filename in filenames:
-                yield self[os.path.relpath(os.path.join(dirpath, filename), self.cachedir)]
+        for key, value in self.items():
+            yield value
 
     def items(self):
-        for dirpath, _, filenames in os.walk(self.cachedir):
-            for filename in filenames:
-                index = os.path.relpath(os.path.join(dirpath, filename), self.cachedir)
-                yield index, self[os.path.relpath(os.path.join(dirpath, filename), self.cachedir)]
+        for key in self.keys():
+            yield key, self[key]
 
     def update(self, d):
-        for k, v in d.items():
+        generator = d.items() if hasattr(d, 'items') else d
+        for k, v in generator:
             self[k] = v
 
     def get(self, index, default = None):
